@@ -17,12 +17,6 @@ class GSFileSystem(ObjectFSWrapper):
     PARAM_CHECKSUM = "etag"
     DETAIL_FIELDS = frozenset(("etag", "size"))
 
-    def __init__(self, **config):
-        super().__init__(**config)
-
-        url = config.get("url", "gs://")
-        self.path_info = self.PATH_CLS(url)
-
     def _prepare_credentials(self, **config):
         login_info = {"consistency": None}
         login_info["project"] = config.get("projectname")
