@@ -17,14 +17,14 @@ from ..base import RemoteCmdError
 try:
     import paramiko
 except ImportError:
-    paramiko = None
+    paramiko = None  # type: ignore
 
 
 logger = logging.getLogger(__name__)
 
 
 def sizeof_fmt(num, suffix="B"):
-    """ Convert number of bytes to human-readable string """
+    """Convert number of bytes to human-readable string"""
     for unit in ["", "K", "M", "G", "T", "P", "E", "Z"]:
         if abs(num) < 1024.0:
             return f"{num:3.1f}{unit}{suffix}"
