@@ -12,10 +12,9 @@ def _update_import_on_remote(stage, remote, jobs):
             "be updated with --to-remote"
         )
 
-    url = stage.deps[0].path_info.url
-    stage.outs[0].hash_info = stage.repo.cloud.transfer(
-        url, jobs=jobs, remote=remote, command="update"
-    )
+    url = stage.deps[0].def_path
+    remote = stage.repo.cloud.get_remote(remote, "update")
+    stage.outs[0].transfer(url, odb=remote.odb, jobs=jobs, update=True)
 
 
 def update_import(stage, rev=None, to_remote=False, remote=None, jobs=None):
